@@ -16,8 +16,8 @@ without any warranty. */
 int
 main(int argc, char **argv)
 {
-  const char *goodTable = "en-us-g1.ctb";
-  const char *badTable = "bad.ctb";
+  const char *goodTable = "tables/en-us-g1.ctb";
+  const char *badTable = "tests/tables/bad.ctb";
   int result = 0;
 
   if (lou_checkTable(goodTable) == 0)
