@@ -17,7 +17,7 @@ main(int argc, char **argv)
 {
   const char* goodTable = "tables/en-us-g1.ctb";
   const char* badTable = "tests/tables/bad.ctb";
-  void* table = NULL;
+  const void* table = NULL;
   int result = 0;
 
   table = lou_getTable(goodTable);
