@@ -6,8 +6,8 @@ $result = 0;
 foreach $file (@data)
 {
 	chomp($file);
-	
-	@output = `../tools/lou_compare -t tables/en-ueb-g2.ctb < $file`;
+
+	@output = `./check_ueb_test_data -t tables/en-ueb-g2.ctb < $file`;
 	if($? != 0)
 	{
 		$result = 1;
