@@ -87,9 +87,8 @@ print_widechars(widechar *buffer, int length) {
 	free(result_buf);
 }
 
-/* direction, 0=forward, 1=backwards, 2=both directions. If diagnostics is 1 then
- * print diagnostics in case where the translation is not as
- * expected */
+/* direction, 0=forward, 1=backwards. If diagnostics is 1 then print
+ * diagnostics in case where the translation is not as expected */
 
 int
 check_base(const char *tableList, const char *input, const char *expected,
@@ -99,7 +98,7 @@ check_base(const char *tableList, const char *input, const char *expected,
 	int direction = in.direction;
 	const int *expected_inputPos = in.expected_inputPos;
 	const int *expected_outputPos = in.expected_outputPos;
-	if (in.direction < 0 || in.direction > 2) {
+	if (in.direction < 0 || in.direction > 1) {
 		fprintf(stderr, "Invalid direction.\n");
 		return 1;
 	}
@@ -123,225 +122,210 @@ check_base(const char *tableList, const char *input, const char *expected,
 				"realInputLength not supported when maxOutputLength is not specified\n");
 		return 1;
 	}
-	while (1) {
-		widechar *inbuf, *outbuf, *expectedbuf;
-		int inlen = parsed_strlen(input);
-		int actualInlen;
-		const int outlen_multiplier = 4 + sizeof(widechar) * 2;
-		int outlen = inlen * outlen_multiplier;
-		int expectedlen = strlen(expected);
-		int funcStatus = 0;
-		formtype *typeformbuf = NULL;
-		int *inputPos = NULL;
-		int *outputPos = NULL;
-		int cursorPos = 0;
-		inbuf = malloc(sizeof(widechar) * inlen);
-		outbuf = malloc(sizeof(widechar) * outlen);
-		expectedbuf = malloc(sizeof(widechar) * expectedlen);
-		if (in.typeform != NULL) {
-			typeformbuf = malloc(outlen * sizeof(formtype));
-			memcpy(typeformbuf, in.typeform, inlen * sizeof(formtype));
+	widechar *inbuf, *outbuf, *expectedbuf;
+	int inlen = parsed_strlen(input);
+	int actualInlen;
+	const int outlen_multiplier = 4 + sizeof(widechar) * 2;
+	int outlen = inlen * outlen_multiplier;
+	int expectedlen = strlen(expected);
+	int funcStatus = 0;
+	formtype *typeformbuf = NULL;
+	int *inputPos = NULL;
+	int *outputPos = NULL;
+	int cursorPos = 0;
+	inbuf = malloc(sizeof(widechar) * inlen);
+	outbuf = malloc(sizeof(widechar) * outlen);
+	expectedbuf = malloc(sizeof(widechar) * expectedlen);
+	if (in.typeform != NULL) {
+		typeformbuf = malloc(outlen * sizeof(formtype));
+		memcpy(typeformbuf, in.typeform, inlen * sizeof(formtype));
+	}
+	if (in.cursorPos >= 0) {
+		cursorPos = in.cursorPos;
+	}
+	if (in.max_outlen >= 0) {
+		outlen = in.max_outlen;
+	}
+	inlen = _lou_extParseChars(input, inbuf);
+	if (!inlen) {
+		fprintf(stderr, "Cannot parse input string.\n");
+		retval = 1;
+		goto fail;
+	}
+	if (in.real_inlen > inlen) {
+		fprintf(stderr,
+				"expected realInputLength (%d) may not exceed total input length "
+				"(%d)\n",
+				in.real_inlen, inlen);
+		return 1;
+	}
+	if (expected_inputPos) {
+		inputPos = malloc(sizeof(int) * outlen);
+	}
+	if (expected_outputPos) {
+		outputPos = malloc(sizeof(int) * inlen);
+	}
+	actualInlen = inlen;
+	// Note that this loop is not strictly needed to make the current tests pass, but
+	// in the general case it is needed because it is theoretically possible that we
+	// provided a too short output buffer.
+	for (int k = 1; k <= 3; k++) {
+		if (direction == 1) {
+			funcStatus = _lou_backTranslate(tableList, in.display_table, inbuf,
+					&actualInlen, outbuf, &outlen, typeformbuf, NULL, outputPos, inputPos,
+					&cursorPos, in.mode, NULL, NULL);
+		} else {
+			funcStatus = _lou_translate(tableList, in.display_table, inbuf, &actualInlen,
+					outbuf, &outlen, typeformbuf, NULL, outputPos, inputPos, &cursorPos,
+					in.mode, NULL, NULL);
 		}
-		if (in.cursorPos >= 0) {
-			cursorPos = in.cursorPos;
-		}
-		if (in.max_outlen >= 0) {
-			outlen = in.max_outlen;
-		}
-		inlen = _lou_extParseChars(input, inbuf);
-		if (!inlen) {
-			fprintf(stderr, "Cannot parse input string.\n");
+		if (!funcStatus) {
+			fprintf(stderr, "Translation failed.\n");
 			retval = 1;
 			goto fail;
 		}
-		if (in.real_inlen > inlen) {
-			fprintf(stderr,
-					"expected realInputLength (%d) may not exceed total input length "
-					"(%d)\n",
-					in.real_inlen, inlen);
-			return 1;
-		}
-		if (expected_inputPos) {
-			inputPos = malloc(sizeof(int) * outlen);
-		}
-		if (expected_outputPos) {
-			outputPos = malloc(sizeof(int) * inlen);
-		}
-		actualInlen = inlen;
-		// Note that this loop is not strictly needed to make the current tests pass, but
-		// in the general case it is needed because it is theoretically possible that we
-		// provided a too short output buffer.
-		for (int k = 1; k <= 3; k++) {
-			if (direction == 1) {
-				funcStatus = _lou_backTranslate(tableList, in.display_table, inbuf,
-						&actualInlen, outbuf, &outlen, typeformbuf, NULL, outputPos,
-						inputPos, &cursorPos, in.mode, NULL, NULL);
-			} else {
-				funcStatus = _lou_translate(tableList, in.display_table, inbuf,
-						&actualInlen, outbuf, &outlen, typeformbuf, NULL, outputPos,
-						inputPos, &cursorPos, in.mode, NULL, NULL);
-			}
-			if (!funcStatus) {
-				fprintf(stderr, "Translation failed.\n");
-				retval = 1;
-				goto fail;
-			}
-			if (in.max_outlen >= 0 || inlen == actualInlen) {
-				break;
-			} else if (k < 3) {
-				// Hm, something is not quite right. Try again with a larger outbuf
-				free(outbuf);
-				outlen = inlen * outlen_multiplier * (k + 1);
-				outbuf = malloc(sizeof(widechar) * outlen);
-				if (expected_inputPos) {
-					free(inputPos);
-					inputPos = malloc(sizeof(int) * outlen);
-				}
-				fprintf(stderr,
-						"Warning: For %s: returned inlen (%d) differs from passed inlen "
-						"(%d) "
-						"using outbuf of size %d. Trying again with bigger outbuf "
-						"(%d).\n",
-						input, actualInlen, inlen, inlen * outlen_multiplier * k, outlen);
-				actualInlen = inlen;
-			}
-		}
-		expectedlen = _lou_extParseChars(expected, expectedbuf);
-		for (i = 0; i < outlen && i < expectedlen && expectedbuf[i] == outbuf[i]; i++)
-			;
-		if (i < outlen || i < expectedlen) {
-			retval = 1;
-			if (in.diagnostics) {
-				outbuf[outlen] = 0;
-				fprintf(stderr, "Input:    '%s'\n", input);
-				/* Print the original typeform not the typeformbuf, as the
-				 * latter has been modified by the translation and contains some
-				 * information about outbuf */
-				if (in.typeform != NULL) print_typeform(in.typeform, inlen);
-				if (in.cursorPos >= 0) fprintf(stderr, "Cursor:   %d\n", in.cursorPos);
-				fprintf(stderr, "Expected: '%s' (length %d)\n", expected, expectedlen);
-				fprintf(stderr, "Received: '");
-				print_widechars(outbuf, outlen);
-				fprintf(stderr, "' (length %d)\n", outlen);
-
-				uint8_t *expected_utf8;
-				uint8_t *out_utf8;
-				size_t expected_utf8_len;
-				size_t out_utf8_len;
-#ifdef WIDECHARS_ARE_UCS4
-				expected_utf8 = u32_to_u8(&expectedbuf[i], 1, NULL, &expected_utf8_len);
-				out_utf8 = u32_to_u8(&outbuf[i], 1, NULL, &out_utf8_len);
-#else
-				expected_utf8 = u16_to_u8(&expectedbuf[i], 1, NULL, &expected_utf8_len);
-				out_utf8 = u16_to_u8(&outbuf[i], 1, NULL, &out_utf8_len);
-#endif
-
-				if (i < outlen && i < expectedlen) {
-					fprintf(stderr,
-							"Diff:     Expected '%.*s' but received '%.*s' in index %d\n",
-							(int)expected_utf8_len, expected_utf8, (int)out_utf8_len,
-							out_utf8, i);
-				} else if (i < expectedlen) {
-					fprintf(stderr,
-							"Diff:     Expected '%.*s' but received nothing in index "
-							"%d\n",
-							(int)expected_utf8_len, expected_utf8, i);
-				} else {
-					fprintf(stderr,
-							"Diff:     Expected nothing but received '%.*s' in index "
-							"%d\n",
-							(int)out_utf8_len, out_utf8, i);
-				}
-				free(expected_utf8);
-				free(out_utf8);
-			}
-		}
-		if (expected_inputPos) {
-			int error_printed = 0;
-			for (i = 0; i < outlen; i++) {
-				if (expected_inputPos[i] != inputPos[i]) {
-					retval = 1;
-					if (in.diagnostics) {
-						if (!error_printed) {  // Print only once
-							fprintf(stderr, "Input position failure:\n");
-							error_printed = 1;
-						}
-						fprintf(stderr, "Expected %d, received %d in index %d\n",
-								expected_inputPos[i], inputPos[i], i);
-					}
-				}
-			}
-		}
-		if (expected_outputPos) {
-			int error_printed = 0;
-			for (i = 0; i < inlen; i++) {
-				if (expected_outputPos[i] != outputPos[i]) {
-					retval = 1;
-					if (in.diagnostics) {
-						if (!error_printed) {  // Print only once
-							fprintf(stderr, "Output position failure:\n");
-							error_printed = 1;
-						}
-						fprintf(stderr, "Expected %d, received %d in index %d\n",
-								expected_outputPos[i], outputPos[i], i);
-					}
-				}
-			}
-		}
-		if ((in.expected_cursorPos >= 0) && (cursorPos != in.expected_cursorPos)) {
-			retval = 1;
-			if (in.diagnostics) {
-				fprintf(stderr, "Cursor position failure:\n");
-				fprintf(stderr, "Initial:%d Expected:%d Actual:%d \n", in.cursorPos,
-						in.expected_cursorPos, cursorPos);
-			}
-		}
-		if (in.max_outlen < 0 && inlen != actualInlen) {
-			retval = 1;
-			if (in.diagnostics) {
-				fprintf(stderr,
-						"Unexpected error happened: input length is not the same before "
-						"as "
-						"after the translation:\n");
-				fprintf(stderr, "Before: %d After: %d \n", inlen, actualInlen);
-			}
-		} else if (actualInlen > inlen) {
-			retval = 1;
-			if (in.diagnostics) {
-				fprintf(stderr,
-						"Unexpected error happened: returned input length (%d) exceeds "
-						"total input length (%d)\n",
-						actualInlen, inlen);
-			}
-		} else if (in.real_inlen >= 0 && in.real_inlen != actualInlen) {
-			retval = 1;
-			if (in.diagnostics) {
-				fprintf(stderr, "Real input length failure:\n");
-				fprintf(stderr, "Expected: %d, received: %d\n", in.real_inlen,
-						actualInlen);
-			}
-		}
-
-	fail:
-		free(inbuf);
-		free(outbuf);
-		free(expectedbuf);
-		free(typeformbuf);
-		free(inputPos);
-		free(outputPos);
-
-		if (direction == 2) {
-			const char *tmp = input;
-			input = expected;
-			expected = tmp;
-			expected_inputPos = in.expected_outputPos;
-			expected_outputPos = in.expected_inputPos;
-			direction = 1;
-			continue;
-		} else {
+		if (in.max_outlen >= 0 || inlen == actualInlen) {
 			break;
+		} else if (k < 3) {
+			// Hm, something is not quite right. Try again with a larger outbuf
+			free(outbuf);
+			outlen = inlen * outlen_multiplier * (k + 1);
+			outbuf = malloc(sizeof(widechar) * outlen);
+			if (expected_inputPos) {
+				free(inputPos);
+				inputPos = malloc(sizeof(int) * outlen);
+			}
+			fprintf(stderr,
+					"Warning: For %s: returned inlen (%d) differs from passed inlen "
+					"(%d) "
+					"using outbuf of size %d. Trying again with bigger outbuf "
+					"(%d).\n",
+					input, actualInlen, inlen, inlen * outlen_multiplier * k, outlen);
+			actualInlen = inlen;
 		}
 	}
+	expectedlen = _lou_extParseChars(expected, expectedbuf);
+	for (i = 0; i < outlen && i < expectedlen && expectedbuf[i] == outbuf[i]; i++)
+		;
+	if (i < outlen || i < expectedlen) {
+		retval = 1;
+		if (in.diagnostics) {
+			outbuf[outlen] = 0;
+			fprintf(stderr, "Input:    '%s'\n", input);
+			/* Print the original typeform not the typeformbuf, as the
+			 * latter has been modified by the translation and contains some
+			 * information about outbuf */
+			if (in.typeform != NULL) print_typeform(in.typeform, inlen);
+			if (in.cursorPos >= 0) fprintf(stderr, "Cursor:   %d\n", in.cursorPos);
+			fprintf(stderr, "Expected: '%s' (length %d)\n", expected, expectedlen);
+			fprintf(stderr, "Received: '");
+			print_widechars(outbuf, outlen);
+			fprintf(stderr, "' (length %d)\n", outlen);
+
+			uint8_t *expected_utf8;
+			uint8_t *out_utf8;
+			size_t expected_utf8_len;
+			size_t out_utf8_len;
+#ifdef WIDECHARS_ARE_UCS4
+			expected_utf8 = u32_to_u8(&expectedbuf[i], 1, NULL, &expected_utf8_len);
+			out_utf8 = u32_to_u8(&outbuf[i], 1, NULL, &out_utf8_len);
+#else
+			expected_utf8 = u16_to_u8(&expectedbuf[i], 1, NULL, &expected_utf8_len);
+			out_utf8 = u16_to_u8(&outbuf[i], 1, NULL, &out_utf8_len);
+#endif
+
+			if (i < outlen && i < expectedlen) {
+				fprintf(stderr,
+						"Diff:     Expected '%.*s' but received '%.*s' in index %d\n",
+						(int)expected_utf8_len, expected_utf8, (int)out_utf8_len,
+						out_utf8, i);
+			} else if (i < expectedlen) {
+				fprintf(stderr,
+						"Diff:     Expected '%.*s' but received nothing in index "
+						"%d\n",
+						(int)expected_utf8_len, expected_utf8, i);
+			} else {
+				fprintf(stderr,
+						"Diff:     Expected nothing but received '%.*s' in index "
+						"%d\n",
+						(int)out_utf8_len, out_utf8, i);
+			}
+			free(expected_utf8);
+			free(out_utf8);
+		}
+	}
+	if (expected_inputPos) {
+		int error_printed = 0;
+		for (i = 0; i < outlen; i++) {
+			if (expected_inputPos[i] != inputPos[i]) {
+				retval = 1;
+				if (in.diagnostics) {
+					if (!error_printed) {  // Print only once
+						fprintf(stderr, "Input position failure:\n");
+						error_printed = 1;
+					}
+					fprintf(stderr, "Expected %d, received %d in index %d\n",
+							expected_inputPos[i], inputPos[i], i);
+				}
+			}
+		}
+	}
+	if (expected_outputPos) {
+		int error_printed = 0;
+		for (i = 0; i < inlen; i++) {
+			if (expected_outputPos[i] != outputPos[i]) {
+				retval = 1;
+				if (in.diagnostics) {
+					if (!error_printed) {  // Print only once
+						fprintf(stderr, "Output position failure:\n");
+						error_printed = 1;
+					}
+					fprintf(stderr, "Expected %d, received %d in index %d\n",
+							expected_outputPos[i], outputPos[i], i);
+				}
+			}
+		}
+	}
+	if ((in.expected_cursorPos >= 0) && (cursorPos != in.expected_cursorPos)) {
+		retval = 1;
+		if (in.diagnostics) {
+			fprintf(stderr, "Cursor position failure:\n");
+			fprintf(stderr, "Initial:%d Expected:%d Actual:%d \n", in.cursorPos,
+					in.expected_cursorPos, cursorPos);
+		}
+	}
+	if (in.max_outlen < 0 && inlen != actualInlen) {
+		retval = 1;
+		if (in.diagnostics) {
+			fprintf(stderr,
+					"Unexpected error happened: input length is not the same before "
+					"as "
+					"after the translation:\n");
+			fprintf(stderr, "Before: %d After: %d \n", inlen, actualInlen);
+		}
+	} else if (actualInlen > inlen) {
+		retval = 1;
+		if (in.diagnostics) {
+			fprintf(stderr,
+					"Unexpected error happened: returned input length (%d) exceeds "
+					"total input length (%d)\n",
+					actualInlen, inlen);
+		}
+	} else if (in.real_inlen >= 0 && in.real_inlen != actualInlen) {
+		retval = 1;
+		if (in.diagnostics) {
+			fprintf(stderr, "Real input length failure:\n");
+			fprintf(stderr, "Expected: %d, received: %d\n", in.real_inlen, actualInlen);
+		}
+	}
+
+fail:
+	free(inbuf);
+	free(outbuf);
+	free(expectedbuf);
+	free(typeformbuf);
+	free(inputPos);
+	free(outputPos);
 
 	return retval;
 }
